@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import Optional, Union
+from typing import Union
 
 from .report import Report
 from .settings import NavEntry, path_to_nav_entry
@@ -32,5 +32,5 @@ def add_pkg_docs(
             )
 
             # now create a new page and add the doc-entry
-            page = report.page(module_nav_entry, append=False)
+            page = report.page(module_nav_entry, truncate=True)
             page.add(page.md.Docstring(".".join(module_path.parts)))
