@@ -1,27 +1,31 @@
 import inspect
 from functools import partial
 from pathlib import Path
-from typing import Callable, Optional, Union
+from typing import Any, Dict
 
-from . import md
+
+def register_md(name):
+    def register_md_named(cls):
+        MdProxy._proxied_classes[name] = cls
+        return cls
+
+    return register_md_named
 
 
 class MdProxy:
-    def __init__(
-        self,
-        store_path: Path,
-        report_path: Path,
-        datatable_id: Optional[Union[str, Callable[[str], str]]] = None,
-        altair_id: Optional[Union[str, Callable[[str], str]]] = None,
-    ):
+    _proxied_classes: Dict[str, Any] = dict()
+
+    def __init__(self, store_path: Path, report_path: Path, javascript_path: Path):
         self.store_path = store_path
         self.report_path = report_path
-        self.datatable_id = datatable_id
-        self.altair_id = altair_id
+        self.javascript_path = javascript_path
 
     def __getattr__(self, name):
         # we are not checking if it is included; if not, should raise error
-        obj = getattr(md, name)
+        if name in self._proxied_classes:
+            obj = self._proxied_classes[name]
+        else:
+            raise AttributeError(f"No MdObj of name '{name}' registered")
 
         # if is a class; try to fix the init method
         if inspect.isclass(obj):
@@ -32,20 +36,8 @@ class MdProxy:
                 partial_kwargs["store_path"] = self.store_path
             if "report_path" in obj_sig.parameters:
                 partial_kwargs["report_path"] = self.report_path
-
-            if (
-                self.datatable_id is not None
-                and name == "DataTable"
-                and "table_id" in obj_sig.parameters
-            ):
-                partial_kwargs["table_id"] = self.datatable_id
-
-            if (
-                self.altair_id is not None
-                and name == "Altair"
-                and "altair_id" in obj_sig.parameters
-            ):
-                partial_kwargs["altair_id"] = self.altair_id
+            if "javascript_path" in obj_sig.parameters:
+                partial_kwargs["javascript_path"] = self.javascript_path
 
             if len(partial_kwargs) > 0:
                 return partial(obj, **partial_kwargs)
@@ -54,3 +46,7 @@ class MdProxy:
 
         else:
             return obj
+
+    @property
+    def proxied_clases(self):
+        return self._proxied_classes
