@@ -6,27 +6,27 @@ responsible for creating a mkdocs project if it doesn't exist
 already and ensuring that the neccessary settings are all 
 included. 
 """
-import contextlib
 import os
 import shutil
-import time
+from contextlib import nullcontext
 from pathlib import Path
-from typing import Any, ContextManager, Dict, Mapping, Optional, Tuple, Union
+from typing import (Any, ContextManager, Dict, Literal, Mapping, Optional,
+                    Tuple, Union)
 
 import yaml
 from frontmatter.default_handlers import DEFAULT_POST_TEMPLATE, YAMLHandler
 from immutabledict import immutabledict
 
-from .counters import Counters
-from .exceptions import (ReportExistsError, ReportNotExistsError,
+from .code_context import CodeContext, Layouts
+from .exceptions import (ContextActiveError, IncorrectSuffixError,
+                         ReportExistsError, ReportNotExistsError,
                          ReportNotValidError, TrackerEmptyError,
                          TrackerIncompleteError)
-from .md import MdObj, Raw, SpacedText, Tab, Text
+from .md import IDStore, MdObj, Raw, SpacedText, Text, merge_settings
 from .md_proxy import MdProxy
-from .settings import (NavEntry, add_nav_entry, load_yaml, merge_settings,
-                       path_to_nav_entry, save_yaml)
-from .stack import Tracker
-from .utils import relative_repo_root
+from .settings import (NavEntry, add_nav_entry, load_yaml, path_to_nav_entry,
+                       save_yaml)
+from .utils import find_comment_ids, relative_repo_root
 
 default_settings = immutabledict(
     {
@@ -106,9 +106,6 @@ class Report:
     def __init__(
         self,
         path: Optional[Union[str, Path]] = None,
-        report_name: Optional[str] = None,
-        exist_ok: bool = True,
-        settings: Optional[Mapping[str, str]] = None,
     ) -> None:
         # need to ensure it is of type Path
         if path is None:
@@ -119,26 +116,15 @@ class Report:
                     "If no report path is given, the 'MKREPORTS_DIR'  environment variable has to be set."
                 )
         self._path = Path(path).absolute()
-        self.report_name = report_name
         # first check if the path exists and is not empty and return error if that is not ok
-        if self.path.exists() and any(self.path.iterdir()):
-            if not exist_ok:
-                raise ReportExistsError(f"{self.path} already exists")
-            else:
-                # nothing to do here
-                pass
-        else:
-            # check if should be created
-            if report_name is not None:
-                # call the functions from mkdocs that creates a new report
-                if settings is None:
-                    settings = default_settings
-                self._create_new(report_name, settings)
-            else:
-                raise ReportNotExistsError(f"{self.path} does not exist.")
-
-        # ensure that all the dependencies are there
-        self._ensure_is_report()
+        if not self.path.exists():
+            raise ReportNotExistsError(f"{self.path} does not exist.")
+        if not self.mkdocs_file.exists() or not self.mkdocs_file.is_file():
+            raise ReportNotValidError(f"{self.mkdocs_file} does not exist")
+        if not self.docs_dir.exists() or not self.docs_dir.is_dir():
+            raise ReportNotValidError(f"{self.docs_dir} does not exist")
+        if not self.index_file.exists() or not self.index_file.is_file():
+            raise ReportNotValidError(f"{self.index_file} does not exist")
 
     @property
     def path(self) -> Path:
@@ -156,31 +142,46 @@ class Report:
     def index_file(self) -> Path:
         return self.docs_dir / "index.md"
 
-    def _create_new(self, site_name: str, settings: Mapping[str, str]) -> None:
-        # create the directoriewwwwwwbhec
-        self.docs_dir.mkdir(exist_ok=True, parents=True)
-        # index.md just remains empty
-        self.index_file.touch()
-        # the settings are our serialized yaml
-        settings = dict(settings.items())  # ensure settings is regular dict
-        settings["site_name"] = site_name
-        with self.mkdocs_file.open("w") as f:
-            yaml.dump(settings, f, Dumper=yaml.Dumper, default_flow_style=False)
+    @property
+    def javascript_path(self) -> Path:
+        return self.docs_dir / "javascript"
+
+    @classmethod
+    def create(
+        cls,
+        path: Union[str, Path],
+        report_name: str,
+        settings: Optional[Mapping[str, str]] = default_settings,
+        exist_ok: bool = False,
+    ) -> "Report":
+        path = Path(path)
+        # create the directory
+        try:
+            (path / "docs").mkdir(exist_ok=exist_ok, parents=True)
+        except FileExistsError:
+            raise ReportExistsError(f"{path / 'docs'} already exists.")
+
+        # index.md created, but done nothing if it exists
+        # if exist_ok=False, the previousalready failed otherwise
+        (path / "docs" / "index.md").touch()
+
+        # only do it if mkdocs_yml does not exist yet
+        mkdocs_file = path / "mkdocs.yml"
+        if not mkdocs_file.exists():
+            # the settings are our serialized yaml
+            # ensure settings is regular dict
+            settings = dict(settings.items()) if settings is not None else {}
+            settings["site_name"] = report_name
+            with (path / "mkdocs.yml").open("w") as f:
+                yaml.dump(settings, f, Dumper=yaml.Dumper, default_flow_style=False)
 
         # also create the overrides doc
-        overrides_dir = self.path / "overrides"
+        overrides_dir = path / "overrides"
         overrides_dir.mkdir(exist_ok=True, parents=True)
         with (overrides_dir / "main.html").open("w") as f:
             f.write(main_html_override)
 
-    def _ensure_is_report(self) -> None:
-        # now ensure that these all exist
-        if not self.mkdocs_file.exists() or not self.mkdocs_file.is_file():
-            raise ReportNotValidError(f"{self.mkdocs_file} does not exist")
-        if not self.docs_dir.exists() or not self.docs_dir.is_dir():
-            raise ReportNotValidError(f"{self.docs_dir} does not exist")
-        if not self.index_file.exists() or not self.index_file.is_file():
-            raise ReportNotValidError(f"{self.index_file} does not exist")
+        return cls(path)
 
     def _add_nav_entry(self, nav_entry) -> None:
         # check that the nav-entry is relative; if absolute,
@@ -197,26 +198,29 @@ class Report:
     def page(
         self,
         page_name: Union[NavEntry, Path, str],
-        append: bool = False,
-        init_counter_time: bool = False,
-        append_code_file: Optional[Union[str, Path]] = None,
+        truncate: bool = False,
+        add_bottom: bool = True,
     ) -> "Page":
         # if the page_name is just a string, we turn it into a dictionary
         # based on the hierarchical names
         if isinstance(page_name, (str, Path)):
             path = Path(page_name)
+            if path.suffix == "":
+                path = path.with_suffix(".md")
             nav_entry = path_to_nav_entry(path)
         else:
             nav_entry = page_name
             path = nav_entry[1]
 
+        if path.suffix == "":
+            path = path.with_suffix(".md")
         if path.suffix != ".md":
             raise ValueError(f"{path} needs to have extension '.md'")
 
         # if the file already exists, just return a 'Page',
         # else create a new nav-entry and the file and return a 'Page'
         if (self.docs_dir / path).exists():
-            if not append:
+            if truncate:
                 # delete the existing site
                 (self.docs_dir / path).unlink()
                 (self.docs_dir / path).touch()
@@ -232,8 +236,7 @@ class Report:
         return Page(
             self.docs_dir / path,
             report=self,
-            init_counter_time=init_counter_time,
-            append_code_file=append_code_file,
+            add_bottom=add_bottom,
         )
 
 
@@ -242,67 +245,80 @@ class Page:
         self,
         path: Path,
         report: Report,
-        page_settings: Optional[Dict[str, Any]] = None,
-        mkdocs_settings: Optional[Dict[str, Any]] = None,
-        init_counter_time: bool = False,
-        append_code_file: Optional[Union[Path, str]] = None,
+        code_layout: Layouts = "tabbed",
+        code_name_only: bool = False,
+        add_bottom: bool = True,
     ) -> None:
         self._path = path.absolute()
-        if init_counter_time:
-            # use time in ms
-            self._counters = Counters(start_with=int(time.time() * 1000))
-        else:
-            self._counters = Counters()
-        self.report = report
+        # check that the file exists and ends with .md
+        if not self.path.exists():
+            raise FileNotFoundError(f"file {self.path} does not exist.")
+        if not self.path.suffix == ".md":
+            raise IncorrectSuffixError(f"file {self.path} does not have suffix '.md'")
 
-        # check if the page already exists
-        if not self._path.exists():
-            # we create the file with the settings
-            self.add(Raw(page_settings=page_settings, mkdocs_settings=mkdocs_settings))
+        # we need to parse the file for ids
+        self._idstore = IDStore(used_ids=find_comment_ids(self.path.read_text()))
+        self.report = report
+        self.add_bottom = add_bottom
+        self.code_layout: Layouts = code_layout
+        self.code_name_only = code_name_only
 
         self._md = MdProxy(
-            store_path=self.gen_asset_path,
+            store_path=self.store_path,
             report_path=self.report.path,
-            datatable_id=lambda hash: self._counters.counted_id(f"datatable-{hash}"),
-            altair_id=lambda hash: self._counters.counted_id(f"altair-{hash}"),
+            javascript_path=self.report.javascript_path,
         )
 
-        self.append_code_file = append_code_file
-
-        # a tracker for tracking code to be printed
-        self.reset_tracker()
+        self.code_context: Optional[CodeContext] = None
 
     def __enter__(self) -> "Page":
+        """
+        Return a copy of the page with a new CodeContext set
+        """
+        if self.code_context is not None and self.code_context.active:
+            raise ContextActiveError("The context manager is already active")
+        if self.code_context is None:
+            self.code_context = CodeContext(
+                layout=self.code_layout,
+                name_only=self.code_name_only,
+                add_bottom=self.add_bottom,
+            )
+        self.code_context.__enter__()
+        return self
+
+    def ctx(
+        self,
+        layout: Optional[Layouts] = None,
+        name_only: Optional[bool] = None,
+        add_bottom: Optional[bool] = None,
+    ) -> "Page":
+        if self.code_context is not None and self.code_context.active:
+            raise ContextActiveError("The context manager is already active")
+        self.code_context = CodeContext(
+            layout=layout if layout is not None else self.code_layout,
+            name_only=name_only if name_only is not None else self.code_name_only,
+            add_bottom=add_bottom if add_bottom is not None else self.add_bottom,
+        )
         return self
 
     def __exit__(self, exc_type, exc_val, traceback) -> None:
-        if self.append_code_file:
-            try:
-                self.add(
-                    self.md.Admonition(
-                        self.md.CodeFile(self.append_code_file),
-                        collapse=True,
-                        kind="info",
-                        title=relative_repo_root(self.append_code_file),
-                    )
-                )
-            except Exception:
-                pass
+        """
+        Exit the code context and add output.
+        """
+        if self.code_context is None:
+            raise Exception("__exit__ called before __enter__")
+        self.code_context.__exit__(exc_type, exc_val, traceback)
+        self._add_to_page(
+            self.code_context.md_obj(javascript_path=self.report.javascript_path)
+        )
+        self.code_context = None
 
     def __getattr__(self, name):
         md_class = self.md.__getattr__(name)
 
         def md_and_add(*args, **kwargs):
-            kwargs_add = {
-                name: value
-                for name, value in kwargs.items()
-                if name in ["add_code", "bottom"]
-            }
-            kwargs_md = {
-                name: value
-                for name, value in kwargs.items()
-                if name not in ["add_code", "bottom"]
-            }
+            kwargs_add = {}
+            kwargs_md = kwargs
 
             # now apply to md
             md_obj = md_class(*args, **kwargs_md)
@@ -310,33 +326,29 @@ class Page:
 
         return md_and_add
 
+    def __copy__(self):
+        cls = self.__class__
+        result = cls.__new__(cls)
+        result.__dict__.update(self.__dict__)
+        return result
+
+    @property
+    def notrack(self) -> ContextManager["Page"]:
+        return nullcontext(self)
+
     @property
     def path(self) -> Path:
         return self._path
 
     @property
-    def gen_asset_path(self) -> Path:
-        return self.path.parent / (self._path.stem + "_gen_assets")
+    def store_path(self) -> Path:
+        return self.path.parent / (self._path.stem + "_store")
 
     def clear(self) -> None:
         """Clear the page markdown file and the generated assets directory."""
-        shutil.rmtree(self.gen_asset_path)
+        shutil.rmtree(self.store_path)
         self.path.unlink()
 
-    def reset_tracker(self) -> None:
-        # note: tracker is designed to be passed outside
-        self.tracker = Tracker(omit_levels=0)
-
-    def track_code(self):
-        self.reset_tracker()
-        return self.tracker
-
-    def track_code_start(self):
-        self.tracker.__enter__()
-
-    def track_code_end(self):
-        self.tracker.__exit__(None, None, None)
-
     def md_code(self, highlight: bool = True, full_filename: bool = False) -> MdObj:
         """Print code as markdown that has been tracked."""
         if self.tracker.ctx_active:
@@ -348,27 +360,43 @@ class Page:
         )
 
     def add(
-        self, item: Union[MdObj, Text], add_code: bool = False, bottom: bool = True
-    ) -> ContextManager["Page"]:
+        self,
+        item: Union[MdObj, Text],
+    ) -> "Page":
         # first ensure that item is an MdObj
         if isinstance(item, str):
             item = Raw(item, dedent=True)
         elif isinstance(item, SpacedText):
             item = Raw(item)
 
-        if add_code:
-            # we wrap it all in a tabs, with one tab the main output, the other
-            # the code;
-            item = Tab(item, title="Content") + Tab(
-                self.md_code(highlight=False), title="Code"
-            )
-            # reset the tracker
-            self.reset_tracker()
+        # if a context-manager is active, pass along the object into there
+        if self.code_context is not None:
+            self.code_context.add(item)
+        else:  # else pass it directly to the page
+            self._add_to_page(item)
 
+        # we return a copy of the page, but with the code context not copied
+        # the copy is therefore a shallow copy
+        # page_copy = copy.copy(self)
+        # page_copy.code_context = None
+        return self
+
+    def _add_to_page(
+        self,
+        item: MdObj,
+    ) -> None:
+        """
+        Read the frontmatter and merge it with the additional settings.
+
+        The reason that we do this separately is a minor issue in the
+        frontmatter library, that filters the newlines at the end of the file.
+        https://github.com/eyeseast/python-frontmatter/issues/87
+        """
         # call the markdown and the backmatter
-        md_text = item.to_md_with_bm(page_path=self.path)
+        md_out = item.to_markdown(page_path=self.path, idstore=self._idstore)
+        md_text = md_out.body + md_out.back
 
-        req = item.req_settings()
+        req = md_out.settings
         if len(req.mkdocs) > 0:
             # merge these things into mkdocs
             # there is not allowed to be a nav here
@@ -379,34 +407,14 @@ class Page:
             mkdocs_settings = merge_settings(mkdocs_settings, req.mkdocs)
             save_yaml(mkdocs_settings, self.report.mkdocs_file)
 
-        if bottom:
-            self._add_to_page(top=None, bottom=md_text, page_settings=req.page)
-        else:
-            self._add_to_page(top=md_text, bottom=None, page_settings=req.page)
-
-        return contextlib.nullcontext(self)
-
-    def _add_to_page(
-        self,
-        top: Optional[SpacedText],
-        bottom: Optional[SpacedText],
-        page_settings: Dict[str, Any],
-    ) -> None:
-        """
-        Read the frontmatter and merge it with the additional settings.
-
-        The reason that we do this separately is a minor issue in the
-        frontmatter library, that filters the newlines at the end of the file.
-        https://github.com/eyeseast/python-frontmatter/issues/87
-        """
         metadata, content = load_page(self.path)
         # we need to read the whole page anyway
-        metadata = merge_settings(metadata, page_settings)
+        metadata = merge_settings(metadata, req.page)
 
-        if top is not None:
-            content = top.format_text("", content) + content
-        if bottom is not None:
-            content = content + bottom.format_text(content, "")
+        if self.add_bottom:
+            content = content + md_text.format_text(content, "")
+        else:
+            content = md_text.format_text("", content) + content
 
         write_page(self.path, metadata, content)
 
